#ifndef lint
static const char rcsid[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/multi_player.c,v 1.9 1999/08/03 11:13:08 tristan Exp tristan $";
static const char copyright[] = "Copyright (c) 1998, MpegTV LLC";
#endif


/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "mtvp/mtvp.h"

#include <X11/Xlib.h>

/* challenge.h not available in public distribution */
#ifdef HANDLE_CHALLENGE
#	include "challenge.h"
#endif

/*--------------------------------------------------------------------------+
| Configuration flags														|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
/* for the msg.h package */
#define MTVP_MSG_PTR (&Msg)

/* maximum number of streams that we want to play at the same time! */
#define MAX_PLAYERS	10

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Globals																	|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Statics																	|
+--------------------------------------------------------------------------*/

/* to communicate with the players */
static MTVP_PLAYER	*Player[MAX_PLAYERS];

static MTVP_PLAYER	*Player_with_audio;

/* for the msg.h package */
static MTVP_MSG_STRUCT Msg;

#if 0 /* Tristan Savatier - Tue Apr 13 23:11:10 PDT 1999 */
//static Bool Must_terminate;
#endif /* 0 - Tristan Savatier - Tue Apr 13 23:11:10 PDT 1999 */

/*--------------------------------------------------------------------------+
| read_and_handle_message_from_player										|
+--------------------------------------------------------------------------*/
static int
read_and_handle_message_from_player(MTVP_PLAYER *player)
{
	int		fd = mtvp_get_notification_fd(player);
	MTVP_PMP_HEADER	msg;		/* to hold message header from Player */ 
	MTVP_PMP_DATA_A2G	data;	/* to hold message data from Player */
	int status;
	
	/*
	 * This  callback is called when there is something available for
	 * reading on Player.receive_channel.msg_pipe[PMP_FD_READ]
	 * (also equal to fd in this routine).
	 *
	 * This routine should read one message from the Player
	 * and take the appropriate action.
	 *
	 * It is mostly a place holder in this example: This sample
	 * application ignores most messages.  It just checks the
	 * API version number and quits if the message A2G_COM_QUIT
	 * is received (which should not happen, by the way!).
	 */
	 
	/* read message from player */
	status = mtvp_read_notification(player, &msg, &data);
	if (status != 0) {
		/* this can happen if the player is killed brutally */
		MTVP_MESSAGE_ERROR(("a2g_read_msg: error reading msg"));
		return -1;
	}

	/* trace message if (Msg_select & MSG_INFO) */
	{
		char buf[MTVP_PMP_PRINT_MSG_BUF_SIZE];
		MTVP_MESSAGE(("read_and_handle_message_from_player: received msg: %s", pmp_print_msg(&msg,  (const char *)&data, buf)));
	}
	
	switch ((MTVP_NOTIFY)msg.type) {
		case MTVP_NOTIFY_QUIT:
			/*
			 * Player sends MTVP_NOTIFY_QUIT after receiving MTVP_COMMAND_QUIT and before exiting.
			 */

			mtvp_destroy_player(player);
			
			return -1;
			
			break;
		case MTVP_NOTIFY_VOH_FEEDBACK:
			switch (data.data_voh_feedback.handler_id) {
#ifdef USE_VOH_X
				case MTVP_VOH_HANDLER_ID_X:
					switch (data.data_voh_x_feedback.data_x.type) {
						case MTVP_VOH_X_FEEDBACK_TYPE_EVENT: {
							XEvent *x_event = &data.data_voh_x_feedback.data_x.data.event;
							
							switch (x_event->type) {
								case ButtonRelease:
									/*
									 * This is a click in the video window of this player.
									 * Tell all other players to mute (since we don't know which
									 * one plays the audio), then tell this player to un-mute.
									 */
									{
										u_int i;
										for (i = 0; i < MAX_PLAYERS; i++) {
											int pfd;
											
											if (Player[i] == NULL) {
												continue;
											}
											pfd = mtvp_get_notification_fd(Player[i]);
								
											if (Player[i] != player && pfd != -1) {
												mtvp_send_command_int(Player[i], MTVP_COMMAND_AUDIO_MUTE, 0, (int)True);
											}
										}
									}
									mtvp_send_command_int(player, MTVP_COMMAND_AUDIO_MUTE, 0, (int)False);
									
									/* note what player is supposed to be playing the audio */
									Player_with_audio = player;
									break;
								default:
									/* ignore other X events */
									break;
							}
							break;
						}
						default:
							break;
					}
					break;
#endif /* USE_VOH_X */
				default:
					/* unknown voh handler! */
					break;
			}
			break;
		case MTVP_NOTIFY_AUDIO_MUTE:
			/*
			 * Player sends MTVP_COMMAND_AUDIO_MUTE when an un-muted player
			 * is not able to open the audio device (e.g. device is busy).
			 */
			if (player == Player_with_audio) {
				/* if that comes from the player that was selected for audio, try again */
				mtvp_send_command_int(player, MTVP_COMMAND_AUDIO_MUTE, 0, (int)False);
			}
			break;
		case MTVP_NOTIFY_CHALLENGE:
			/*
			 * This is a reserved message of the API.
			 * MTVP_NOTIFY_CHALLENGE is a private authentification request
			 * which is sent only if the player is started with option -k
			 * or cannot find a valid license key.
			 * The player does not need a valid license key if the front-end
			 * can be authentified. Authentification is done by responding
			 * properly to challenge messages.
			 *
			 * If the player finds a valid license key, it does not
			 * send challenge messages.
			 *
			 * Front-ends who receive challenge messages but are not capable of
			 * dealing with them should advise the user that the player has no
			 * license and will not be functionnal.
			 */

#ifdef HANDLE_CHALLENGE
			if (handle_challenge(player, &msg, &data) != 0) {
				MTVP_FATAL_ERROR(("cannot handle challenge"));
			}
#else
			printf("This front-end is not authentified by mtvp.\nYou need a development license key (in file mtvp.key).\nSee www.mpegtv.com/sdk.html for details.\n");

#endif
			break;
		default:
			/* ignore all other messages */
			break;
	}

	return 0;
}


/*--------------------------------------------------------------------------+
| clean_exit																|
+--------------------------------------------------------------------------*/
static void
clean_exit(int n)
{	
	MTVP_MESSAGE(("(pid=%u): exiting, status = %d", (u_int)getpid(), n));

	/* terminate (real exit this time) */
	exit(n);
}

#if 0 /* Tristan Savatier - Tue Apr 13 23:11:02 PDT 1999 */
///*--------------------------------------------------------------------------+
//| handler_exit																|
//+--------------------------------------------------------------------------*/
///*ARGSUSED*/
//static void
//handler_exit(
//	int     sig
//)
//{
//	pid_t pid = getpid();			/* pid of current process */
//
//	WARNING(("(pid=%u): received signal %d (aborting)", (u_int)pid, sig));
//
//	Must_terminate = True;
//}
#endif /* 0 - Tristan Savatier - Tue Apr 13 23:11:02 PDT 1999 */

/*--------------------------------------------------------------------------+
| start_player																|
+--------------------------------------------------------------------------*/
static void
start_player(MTVP_PLAYER **player, const char **argv)
{
	int status;
	static const char *mtvp_argv[] = {
#ifdef HANDLE_CHALLENGE
		"mtvp",	/* name of the player (if no / in pathname, will search in PATH) */
#else
		"../../bin/mtvp",	/* name of the player (if no / in pathname, will search in PATH) */
#endif
		/* "-a0", */	/* possible options */
		/* "-w", */		/* possible options */
#ifdef HANDLE_CHALLENGE
		"-k",	/* use challenge authentification only, do not look for key file */
#endif
		NULL,	/* must be NULL-terminated */
	};

	/*
	 * First parameter of g2a_init() is a ptr to a NULL-terminated array
	 * of strings.
	 *
	 * The first string is the name of the Player ("mtvp").
	 * (when using libmtvpnew.so, this string is ignored!)
	 * The following strings are possible parameters.
	 *
	 * If the first string is a full pathname (starting with "/")
	 * g2a_init() will run this program as "the Player".
	 *
	 * If the first string is not a full pathname (starting with "/")
	 * g2a_init() will look for the Player in the following places:
	 * - in the directory where argv[0] is located (if argv[0] contains a '/')
	 * - in the current directory (".")
	 * - in the PATH
	 *
	 * Typically first string is "mtvp", if the player was installed
	 * under this name.
	 *
	 * Second parameter of g2a_init() is a string that should be
	 * the argv[0] from the calling program.
	 */

	status = mtvp_create_player(player, mtvp_argv, argv[0], NULL);
	if (status != 0) {
		MTVP_FATAL_ERROR(("cannot start player (%s)", mtvp_argv[0]));
	}
}

/*--------------------------------------------------------------------------+
| main																		|
+--------------------------------------------------------------------------*/
/*ARGSUSED*/
int
main(
	int argc,
	const char **argv,
	const char **envp
)
{
	const char	**p, **q;
	int		n;

	/*
	 * Set Msg.select == -1 to print all the warnings and messages.
	 * See msg.h for the MESSAGE() and WARNING() macros definitions.
	 */
	 
	/* Msg.select = -1; */
	
	MTVP_MESSAGE(("started, pid = %u", (u_int)getpid()));
	
	if (argc <= 1) {
		fprintf(stderr, "usage: multi_player file1.mpg [file2.mpg...]\n");
		fprintf(stderr, "click on window to get corresponding audio\n");
		clean_exit(EXIT_FAILURE);		
	}

	/*
	 * Start a player for each of the files specified, and tell the player
	 * to play the file in loop.
	 */
	
	{
		u_int	i;
		
		for (i = 0; i < MAX_PLAYERS; i++) {

			if (i <= (u_int)(argc - 2)) {
			
				/* start a player */
				start_player(&Player[i], argv);
				
				/* tell player to loop when end of file will be reached */
				mtvp_send_command_int(Player[i], MTVP_COMMAND_LOOP, 0, (int)True);
				
				/*
				 * Tell player to pass us buttons events.  When a click occurs
				 * in a video window, we will tell the corresponding player to
				 * play the audio.
				 */
				{
					MTVP_VOH_X_CNTL data_x;
					
					data_x.handler_id = MTVP_VOH_HANDLER_ID_X;
					data_x.type = MTVP_VOH_X_CNTL_TYPE_SET_EVENT_MASK;
					data_x.data.event_mask = ButtonPressMask|ButtonReleaseMask;
			
					mtvp_send_command_custom(Player[i], MTVP_COMMAND_VOH_CNTL, 0, (const void *)&data_x, sizeof data_x);
				}

				/* tell player to open file */
				mtvp_send_command_string(Player[i], MTVP_COMMAND_OPEN_STREAM_URL, 0, argv[i + 1]);
			}
		}
	}
			
	MTVP_MESSAGE(("entering event loop"));

	/* enter event loop */
	while (True) {
		u_int	i;
		fd_set	rfds;
		int		ret;
		int		maxfd = -1;

		/* initialize rfds structures with players fd */
		FD_ZERO(&rfds);
		for (i = 0; i < MAX_PLAYERS; i++) {
			int fd;
			
			if (Player[i] != NULL) {
				fd = mtvp_get_notification_fd(Player[i]);
			} else {
				continue;
			}
			
			if (fd == -1) {
				continue;
			}
			FD_SET(fd, &rfds);
			/* remember the largest of the fd's */
			if (fd > maxfd) {
				maxfd = fd;
			}
		}
		if (maxfd == -1) {
			/* no players ? exit loop! */
			break;
		}

		/* block until data is available on one of the players fd's */
		if ((ret = select(maxfd + 1, &rfds, NULL, NULL, NULL /* &tv */)) == -1) {
			if (errno == EINTR) {
#if 0 /* Tristan Savatier - Tue Apr 13 20:54:06 PDT 1999 */
//				if (!Must_terminate) {	/* ignore this */
//					continue;
//				}
#endif /* 0 - Tristan Savatier - Tue Apr 13 20:54:06 PDT 1999 */
				continue;
			} else {
			int status = errno;
				MTVP_FATAL_ERROR(("select failed"));
			}
		}
		/* read the player's message on each fd where a message is available */
		for (i = 0; i < MAX_PLAYERS; i++) {
			int fd;
			
			if (Player[i] == NULL) {
				continue;
			}

			fd = mtvp_get_notification_fd(Player[i]);
			
			if (fd == -1) {
				continue;
			}
#if 0 /* Tristan Savatier - Tue Apr 13 20:53:49 PDT 1999 */
//			if (Must_terminate) {
//			
//				g2a_send_void(Player[i], COM_QUIT, 0);
//				
//			} else
#endif /* 0 - Tristan Savatier - Tue Apr 13 20:53:49 PDT 1999 */
			
			if (FD_ISSET(fd, &rfds)) {

				/* read the player's message on each fd where a message is available */
				
				if (read_and_handle_message_from_player(Player[i]) != 0) {
					/* player probably dead, stop listening to this fd! */
					Player[i] = NULL;
				}
			}
		}
	}

	/* terminated successfully */
	clean_exit(EXIT_SUCCESS);

	/* so that lint does not complain! */
	return 0;
}


/* eof */
