#ifndef lint
static const char rcsid[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/simple_player_motif.c,v 1.19 1999/08/03 11:11:51 tristan Exp tristan $";
static char copyright[] = "Copyright (c) 1997, MpegTV LLC";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

/*
 * Motif interface:
 * Copyright (c) 1997, Alain Jobart (aj@bok.net)
 */


#include "mtvp/mtvp.h"

#include <Xm/MainW.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/FileSB.h>

/*--------------------------------------------------------------------------+
| Configuration flags														|
+--------------------------------------------------------------------------*/
/*
 * Define TEST_SIH only to test the sample SIH plugin.
 * Value of TEST_SIH is path of the sample SIH dynamic library.
 * The sample SIH plugin handles URLs of the form "file:<filename>".
 * Try: $ simple_player file:random.mpg (assuming random.mpg is an MPEG
 * file located in the current directory).
 */
// #define TEST_SIH	"../sih/sample_sih.so"

/*--------------------------------------------------------------------------+
| Macros                                                                    |
+--------------------------------------------------------------------------*/
/* for the msg.h package */
#define MTVP_MSG_PTR (&Msg)

/*--------------------------------------------------------------------------+
| Data types                                                                |
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Globals                                                                    |
+--------------------------------------------------------------------------*/

/* motif variables */
Widget Toplevel;
XtAppContext App_context;

Widget File_window = NULL;

XtInputId Input_callback;

/*--------------------------------------------------------------------------+
| Statics                                                                    |
+--------------------------------------------------------------------------*/

/* Default starting ressources of the application */
static char *app_resources[] =
{
    "*background:      grey",
    "*foreground:      black",
    "*text.background: whitesmoke",
    "*shadowThickness: 1",
    NULL
};

/* to communicate with player */
static MTVP_PLAYER    *Player;

/* for the msg.h package */
static MTVP_MSG_STRUCT Msg;

/*--------------------------------------------------------------------------+
| clean_exit                                                                |
+--------------------------------------------------------------------------*/
static void
clean_exit(n)
    int        n;
{    
    MTVP_MESSAGE(("(pid=%u): exiting, status = %d", (u_int)getpid(), n));
    
    /* terminate (real exit this time) */
    exit(n);
}

/*--------------------------------------------------------------------------+
| quit
+--------------------------------------------------------------------------*/
static void 
quit()
{
	/* remove this callback (g2a_cleanup will close this pipe) */
   	XtRemoveInput (Input_callback);

	/* mtvp_destroy_player() should be called to close pipes to the (now defunct) player */
	mtvp_destroy_player(Player);
	
	clean_exit(EXIT_SUCCESS);
}

/*--------------------------------------------------------------------------+
| read_command                                                            |
+--------------------------------------------------------------------------*/
static void
read_command(cd, fd, id)
    XtPointer  cd;
    int        *fd;
    XtInputId  *id;
{
    MTVP_PMP_HEADER    msg;
    MTVP_PMP_DATA_A2G    data;
    int		status;
    
	/*
	 * This  callback is called when there is something available for
	 * reading on Player.receive_channel.msg_pipe[MTVP_PMP_FD_READ]
	 * (also equal to fd in this routine).
	 *
	 * This routine should read one message from the Player
	 * and take the appropriate action.
	 *
	 * It is mostly a place holder in this example: This sample
	 * application ignores most messages.  It just checks the
	 * API version number and quits if the message MTVP_NOTIFY_QUIT
	 * is received (which should not happen, by the way!).
	 */
	 
	/* read message from player */
	status = mtvp_read_notification(Player, &msg, &data);
	if (status != 0) {
		/* this can happen if the player is killed brutally */
		MTVP_MESSAGE_ERROR(("a2g_read_msg: error reading msg"));
		quit();
	}

	{
		char buf[MTVP_PMP_PRINT_MSG_BUF_SIZE];
		MTVP_MESSAGE(("read_command: received msg: %s", pmp_print_msg(&msg,  (const char *)&data, buf)));
	}
    
    switch ((MTVP_NOTIFY)msg.type) {
		case MTVP_NOTIFY_STATE:
			/* show Player's state in the form title */
			switch (data.data_generic.data_int) {
			case MTVP_STATE_PAUSED_AT_END:
				XtVaSetValues(Toplevel, XmNtitle, "Simple Player (Paused)", NULL);
				break;
			case MTVP_STATE_PAUSED:
				XtVaSetValues(Toplevel, XmNtitle, "Simple Player (Paused)", NULL);
				break;
			case MTVP_STATE_PLAYING:
				XtVaSetValues(Toplevel, XmNtitle, "Simple Player (Playing)", NULL);
				break;
			default:
				MTVP_FATAL(("unknow State (should not happen!)"));
			}
			break;
#ifdef TEST_SIH
		case MTVP_NOTIFY_SIH_FEEDBACK:
			{
				u_int payload_size = msg.size - sizeof(int);
				const char *payload_data = (const char *)&data + sizeof(int);
				int	handler_id = data.data_generic.data_int;
				
				printf("received MTVP_NOTIFY_SIH_FEEDBACK from handler_id = %d, size = %u, data = \"%s\"\n", handler_id, payload_size, payload_data);
			}
			break;
		case MTVP_NOTIFY_RETURN_FAILURE:
			/*
			 * This message is received each time a command that was sent with
			 * the MTVP_PMP_FLAG_RETURN_STATUS flag failed.
			 *
			 * This is convenient to test the return status of commands like
			 * MTVP_COMMAND_SIH_LINK or MTVP_COMMAND_OPEN_STREAM_URL.
			 *
			 * Note that if MTVP_PMP_FLAG_RETURN_ORIGINAL
			 * was set too, a copy of the original message is sent back right
			 * after the MTVP_NOTIFY_RETURN_FAILURE message.
			 *
			 * Here, in this example, we know that the only command sent with
			 * MTVP_PMP_FLAG_RETURN_STATUS is MTVP_COMMAND_SIH_LINK. Ideally we should check
			 * what was the command associated with this returned status message.
			 */
			errno = data.data_return_failure.error;
			MTVP_FATAL_ERROR(("cannot link SIH plugin"));
			break;
		case MTVP_NOTIFY_RETURN_SUCCESS:
			/*
			 * This message is received each time a command that was sent with
			 * the MTVP_PMP_FLAG_RETURN_STATUS flag succeded.
			 *
			 * Here, in this example, we know that the only command sent with
			 * MTVP_PMP_FLAG_RETURN_STATUS is MTVP_COMMAND_SIH_LINK. Ideally we should check
			 * what was the command associated with this returned status message.
			 */
			{
				int	handler_id = data.data_return_success.value;
				printf("SIH plugin linked successfully, handler_id = %d\n", handler_id);
				{
					struct {
						int	sih_handler_id;
						char buf[256];
					} sih_cntl_data;
					
					sih_cntl_data.sih_handler_id = handler_id;
					strcpy(sih_cntl_data.buf, "testing MTVP_COMMAND_SIH_CNTL");
					mtvp_send_command_custom(Player, MTVP_COMMAND_SIH_CNTL, 0, (const void *)&sih_cntl_data, sizeof(int) + strlen(sih_cntl_data.buf) + 1);
				}
			}
			break;
#endif /* TEST_SIH */
        case MTVP_NOTIFY_QUIT:
			/*
			 * mtvp sends MTVP_NOTIFY_QUIT after receiving MTVP_COMMAND_QUIT and before exiting.
			 * we will terminate when we receive the SIGCHLD just after mtvp has exited.
			 */

			quit();
 
            break;
        default:
            /* msgs not understood should be ignored */
            break;
    }
}

/*--------------------------------------------------------------------------+
| open_url																	|
+--------------------------------------------------------------------------*/
void
open_url(const char *url)
{
	int		status;
	void	*value;
	
	mtvp_send_command_string(Player, MTVP_COMMAND_OPEN_STREAM_URL, MTVP_PMP_FLAG_SYNCHRONOUS, url);
	if (mtvp_get_command_status(Player, &status, &value) != 0) {
		abort();	// last command did not have MTVP_PMP_FLAG_SYNCHRONOUS!
	}
	
	if (status != 0) {
		printf("%s: %s\n", url, msg_error_string(status));
	} else {
		printf("%s was opened by SIH header-id = %d\n", url, (int)value);
	}
}

/*--------------------------------------------------------------------------+
| OpenOkCB
+--------------------------------------------------------------------------*/
void OpenOkCB (Widget w, XtPointer cd, XtPointer call_data)
{
    /* get the file */
    XmFileSelectionBoxCallbackStruct *cbs =
        (XmFileSelectionBoxCallbackStruct *) call_data;
    char *filename = NULL;
    XmStringGetLtoR (cbs->value, XmSTRING_DEFAULT_CHARSET, &filename);

    /* do the regular stuff */
//    mtvp_send_command_string(Player, MTVP_COMMAND_OPEN_STREAM_URL, 0, filename);
    open_url(filename);

    /* and free the string */
    XtFree (filename);
    XtUnmanageChild (File_window);
}

/*--------------------------------------------------------------------------+
| OpenCancelCB
+--------------------------------------------------------------------------*/
void OpenCancelCB (Widget w, XtPointer cd, XtPointer cbs)
{
    XtUnmanageChild (File_window);
}

/*--------------------------------------------------------------------------+
| OpenCB
+--------------------------------------------------------------------------*/
void OpenCB (Widget w, XtPointer cd, XtPointer cbs)
{
    if (File_window == NULL) {
        Widget help_button;
        XmString t;

        File_window = XmCreateFileSelectionDialog (Toplevel, "Open File",
                                                   NULL, 0);

        XtAddCallback (File_window, XmNokCallback, OpenOkCB, NULL);
        XtAddCallback (File_window, XmNcancelCallback, OpenCancelCB, NULL);

        /* get the help button and un-manage it */
        help_button = XtNameToWidget (File_window, "Help");
        if (help_button != NULL) XtUnmanageChild (help_button);

        /* set the file selection filter */
        t = XmStringCreateLocalized ("*.mpg");
        XmFileSelectionDoSearch (File_window, t);
        XmStringFree (t);

        /* and manage the box */
        XtManageChild (File_window);
        XtPopup (XtParent (File_window), XtGrabNone);
    } else {
        XtManageChild (File_window);
    }
}

/*--------------------------------------------------------------------------+
| PlayCB
+--------------------------------------------------------------------------*/
void PlayCB (Widget w, XtPointer cd, XtPointer cbs)
{
	/* tell Player to start playing */
    mtvp_send_command_int(Player, MTVP_COMMAND_STATE, 0, MTVP_STATE_PLAYING);
}

/*--------------------------------------------------------------------------+
| StepCB
+--------------------------------------------------------------------------*/
void StepCB (Widget w, XtPointer cd, XtPointer cbs)
{
 	/* tell Player to Step one frame */
	/* Note: if player was playing, state changes automatically to MTVP_STATE_PAUSED */
	mtvp_send_command_int(Player, MTVP_COMMAND_STEPS, 0, 1);
}

/*--------------------------------------------------------------------------+
| RewindCB
+--------------------------------------------------------------------------*/
void RewindCB (Widget w, XtPointer cd, XtPointer cbs)
{
	/* tell Player to Rewind */
    mtvp_send_command_int(Player, MTVP_COMMAND_SEEK, 0, 0);
}

/*--------------------------------------------------------------------------+
| QuitCB
+--------------------------------------------------------------------------*/
void QuitCB (Widget w, XtPointer cd, XtPointer cbs)
{
	/*
	 * Tell Player to terminate. You can do that by sending
	 * the MTVP_COMMAND_QUIT command to the player. You could also
	 * send the SIGTERM signal to the process (pid = Player.pid).
	 *
	 * Actually, since clean_exit does send the SIGTERM to the
	 * player, we really do not need to send the MTVP_COMMAND_QUIT, but
	 * it does not hurt...
	 */
    mtvp_send_command_void(Player, MTVP_COMMAND_QUIT, 0);

	/*
	 * We will terminate when we receive the SIGCHLD just after mtvp has exited.
	 */
}

/*--------------------------------------------------------------------------+
| init_form                                                                |
+--------------------------------------------------------------------------*/
static void
init_form(argc, argv)
    int        argc;
    const char **argv;
{    
    Widget main_w, main_form, w;

    /* Open the X connexion */
    XtSetLanguageProc (NULL, NULL, NULL);
    Toplevel = XtVaAppInitialize (&App_context, "MpegTV Motif Front-end",
                                  NULL, 0, &argc, argv,
                                  app_resources, NULL, 0);

    main_w = XtVaCreateWidget ("main_w", xmMainWindowWidgetClass, Toplevel,
                               NULL);
    
    main_form = XtVaCreateWidget ("form", xmFormWidgetClass, main_w,
                                  XmNfractionBase, 5,
                                  XmNhorizontalSpacing, 5,
                                  XmNverticalSpacing, 5,
                                  NULL);
    
    /* Create the open push button */
    w = XtVaCreateManagedWidget ("Open", xmPushButtonWidgetClass, main_form,
                                 XmNtopAttachment, XmATTACH_FORM,
                                 XmNbottomAttachment, XmATTACH_FORM,
                                 XmNleftAttachment, XmATTACH_FORM,
                                 XmNrightAttachment, XmATTACH_POSITION,
                                 XmNrightPosition, 1,
                                 XmNmarginHeight, 10,
                                 XmNmarginWidth, 20,
                                 NULL);
    XtAddCallback (w, XmNactivateCallback, OpenCB, NULL);

    /* Create the play push button */
    w = XtVaCreateManagedWidget ("Play", xmPushButtonWidgetClass, main_form,
                                 XmNtopAttachment, XmATTACH_FORM,
                                 XmNbottomAttachment, XmATTACH_FORM,
                                 XmNleftAttachment, XmATTACH_POSITION,
                                 XmNleftPosition, 1,
                                 XmNrightAttachment, XmATTACH_POSITION,
                                 XmNrightPosition, 2,
                                 NULL);
    XtAddCallback (w, XmNactivateCallback, PlayCB, NULL);

    /* Create the Step push button */
    w = XtVaCreateManagedWidget ("Step", xmPushButtonWidgetClass, main_form,
                                 XmNtopAttachment, XmATTACH_FORM,
                                 XmNbottomAttachment, XmATTACH_FORM,
                                 XmNleftAttachment, XmATTACH_POSITION,
                                 XmNleftPosition, 2,
                                 XmNrightAttachment, XmATTACH_POSITION,
                                 XmNrightPosition, 3,
                                 NULL);
    XtAddCallback (w, XmNactivateCallback, StepCB, NULL);

    /* Create the Rewind push button */
    w = XtVaCreateManagedWidget ("Rewind", xmPushButtonWidgetClass, main_form,
                                 XmNtopAttachment, XmATTACH_FORM,
                                 XmNbottomAttachment, XmATTACH_FORM,
                                 XmNleftAttachment, XmATTACH_POSITION,
                                 XmNleftPosition, 3,
                                 XmNrightAttachment, XmATTACH_POSITION,
                                 XmNrightPosition, 4,
                                 NULL);
    XtAddCallback (w, XmNactivateCallback, RewindCB, NULL);

    /* Create the Quit push button */
    w = XtVaCreateManagedWidget ("Quit", xmPushButtonWidgetClass, main_form,
                                 XmNtopAttachment, XmATTACH_FORM,
                                 XmNbottomAttachment, XmATTACH_FORM,
                                 XmNleftAttachment, XmATTACH_POSITION,
                                 XmNleftPosition, 4,
                                 XmNrightAttachment, XmATTACH_FORM,
                                 NULL);
    XtAddCallback (w, XmNactivateCallback, QuitCB, NULL);

    /* add the io callback */
    Input_callback = XtAppAddInput (App_context, mtvp_get_notification_fd(Player),
                   (XtPointer) XtInputReadMask, read_command, NULL);

    /* display the widgets */
    XtManageChild (main_form);
    XtManageChild (main_w);
    XtRealizeWidget (Toplevel);
    
}



#if 0 /* Tristan Savatier - Mon Aug 2 22:32:08 PDT 1999 */
///*--------------------------------------------------------------------------+
//| handler_sigchld                                                            |
//+--------------------------------------------------------------------------*/
///*ARGSUSED*/
//static void
//handler_sigchld(sig)
//    int     sig;
//{
//	/*
//	 * Note: If you want to call any Xlib or Motif routine from
//	 * a signal handler, you must use XtAppAddWorkProc()
//	 * to handle signals in a Motif-based application, so that
//	 * the signal handler will be executed in the proper context
//	 * in the Motif event loop.
//	 * Please refer to the Motif manual.
//	 *
//	 * This particular signal handler does not
//	 * call any Xlib or Motif routine, so it is safe
//	 * to install it with signal() or sigaction().
//	 */
//
//	MTVP_MESSAGE(("(pid=%u): received signal %d, child process %u died, exiting...", (u_int)getpid(), sig, (u_int)Player.pid));
//	clean_exit(EXIT_FAILURE);
//}
#endif /* 0 - Tristan Savatier - Mon Aug 2 22:32:08 PDT 1999 */


/*--------------------------------------------------------------------------+
| main                                                                        |
+--------------------------------------------------------------------------*/
/*ARGSUSED*/
int
main(argc, argv, envp)
    int argc;
    const char **argv, **envp;
{
    const char    **p, **q;
    int        n;

    /*
     * Set Msg_select to print all the warnings and messages.
     * Warning: if you set Msg_select before calling g2a_init(), you'll
     * also get all the messages in the player, since g2a_init() does
     * a fork and the player uses the same variable.
     */
     
    /* Msg_select = -1; */
    
#if 0 /* Tristan Savatier - Mon Aug 2 22:31:18 PDT 1999 */
//	{
//		struct sigaction sigact;
//		
//		sigact.sa_handler = handler_sigchld;
//		sigemptyset(&sigact.sa_mask);
//		sigact.sa_flags = SA_NOCLDSTOP;
//		if (sigaction(SIGCHLD, &sigact, NULL) != 0) {
//			MTVP_FATAL_ERROR(("can't set handler for SIGCHLD"));
//		}
//	}
#endif /* 0 - Tristan Savatier - Mon Aug 2 22:31:18 PDT 1999 */

    MTVP_MESSAGE(("started, pid = %u", (u_int)getpid()));
    
    /*
     * Start the player
     */
    
    {
    	int		status;
        static const char *mtvp_argv[] = {
#ifdef HANDLE_CHALLENGE
			"mtvp",	/* name of the player (if no / in pathname, will search in PATH) */
#else
			"../../bin/mtvp",	/* name of the player (if no / in pathname, will search in PATH) */
#endif
            /* "-w", */    /* possible options */
            NULL,    /* must be NULL-terminated */
        };

        /*
         * First parameter of g2a_init() is a ptr to a NULL-ternimated array
         * of strings.
         * The first string is the name of the Player ("mtvp").
         * The following strings are possible parameters.
         * If the first string is a full pathname (starting with "/")
         * g2a_init() will run this program as "the Player".
         *
         * If the first string is not a full pathname (starting with "/")
         * g2a_init() will look for the Player in the following places:
         * - in the directory where argv[0] is located (if argv[0] contains a '/')
         * - in the current directory (".")
         * - in the PATH
         *
         * Typically first string is "mtvp", if the player was installed
         * under this name.
         *
         * Second parameter of g2a_init() is a string that should be
         * the argv[0] from the calling program.
         */
    
        if ((status = mtvp_create_player(&Player, mtvp_argv, argv[0], NULL)) != 0) {
			if (status == -1) {
				MTVP_FATAL_ERROR(("cannot start player (%s) - probably mtvp API version mismatch", mtvp_argv[0]));
			} else if (status == EPIPE) {
				MTVP_FATAL_ERROR(("cannot start player (%s) - check that player is in your PATH!", mtvp_argv[0]));
			} else {
				MTVP_FATAL_ERROR(("cannot start player (%s) - could not fork", mtvp_argv[0]));
			}
        }
    }
    
	/* initialize the form */
	init_form(argc, argv);
	
#ifdef TEST_SIH
	/* link custom stream interface */
	mtvp_send_command_string(Player, MTVP_COMMAND_SIH_LINK, MTVP_PMP_FLAG_RETURN_STATUS, TEST_SIH);
#endif

	/* if one argument was passed, assume that it is a URL to open */
	if (argc == 2) {
		/* open stream from URL */
//		mtvp_send_command_string(Player, MTVP_COMMAND_OPEN_STREAM_URL, 0, argv[1]);
    	open_url(argv[1]);
	} else if (argc > 2) {
		fprintf(stderr, "usage: simple_player_motif [URL]\n");
		clean_exit(EXIT_FAILURE);		
	}

    /* main event loop */
    MTVP_MESSAGE(("entering event loop"));
    XtAppMainLoop (App_context);
    
    MTVP_MESSAGE(("What?!? XtAppMainLoop() returned! (should never happen)"));

    /* terminated successfully */
    clean_exit(EXIT_SUCCESS);

    /* so that lint does not complain! */
    return 0;
}


/* eof */
